#include "StdAfx.h"
#include "Resource.h"
#include "List"
#include "Math.h"
#include ".\world.h"

CWorld::CWorld(void)
{
	srand(RAND_MAX);
}

CWorld::~CWorld(void)
{
}

void CWorld::OnDraw(CDC *pDC)
{
	DrawMap(pDC);
	DrawJuncsBG(pDC);

	DrawCars(pDC);

	DrawJuncsTL(pDC);
}

void CWorld::OnTimer()
{
	for(int i = 0; i < automobiles.size(); i++)
	{
		automobiles[i].Main(EVENT_CLOCK);
	}

	for(int i = 0; i < juncs.size(); i++)
	{
		static int k = 0;

		k++;
		if(k == 30)
		{
			juncs[i].Main(EVENT_CLOCK);
			k = 0;
		}
	}
}

void CWorld::Init(CString worldName)
{
	CStdioFile file(worldName, CStdioFile::modeRead);

	CString mapName;
	file.ReadString(mapName);
	LoadMap(mapName);

	CString juncsName;
	file.ReadString(juncsName);
	LoadJuncs(juncsName);

	LoadCars("Car1");

	AddCars();
}

void CWorld::LoadCars(CString carName)
{
	for(int i = 0; i < 1; i++)
	{
		LoadBitmapByName("Car1 sn.bmp", &carBMP[i][DIRECTION_N][0][0]);
		LoadBitmapByName("Car1 sn r.bmp", &carBMP[i][DIRECTION_N][1][0]);
		LoadBitmapByName("Car1 sn l.bmp", &carBMP[i][DIRECTION_N][2][0]);
		LoadBitmapByName("Car1 sn rr.bmp", &carBMP[i][DIRECTION_N][0][1]);
		LoadBitmapByName("Car1 sn ll.bmp", &carBMP[i][DIRECTION_N][0][2]);
		LoadBitmapByName("Car1 sn r rr.bmp", &carBMP[i][DIRECTION_N][1][1]);
		LoadBitmapByName("Car1 sn l ll.bmp", &carBMP[i][DIRECTION_N][2][2]);

		LoadBitmapByName("Car1 ew.bmp", &carBMP[i][DIRECTION_W][0][0]);
		LoadBitmapByName("Car1 ew r.bmp", &carBMP[i][DIRECTION_W][1][0]);
		LoadBitmapByName("Car1 ew l.bmp", &carBMP[i][DIRECTION_W][2][0]);
		LoadBitmapByName("Car1 ew rr.bmp", &carBMP[i][DIRECTION_W][0][1]);
		LoadBitmapByName("Car1 ew ll.bmp", &carBMP[i][DIRECTION_W][0][2]);
		LoadBitmapByName("Car1 ew r rr.bmp", &carBMP[i][DIRECTION_W][1][1]);
		LoadBitmapByName("Car1 ew l ll.bmp", &carBMP[i][DIRECTION_W][2][2]);

		LoadBitmapByName("Car1 ns.bmp", &carBMP[i][DIRECTION_S][0][0]);
		LoadBitmapByName("Car1 ns r.bmp", &carBMP[i][DIRECTION_S][1][0]);
		LoadBitmapByName("Car1 ns l.bmp", &carBMP[i][DIRECTION_S][2][0]);
		LoadBitmapByName("Car1 ns rr.bmp", &carBMP[i][DIRECTION_S][0][1]);
		LoadBitmapByName("Car1 ns ll.bmp", &carBMP[i][DIRECTION_S][0][2]);
		LoadBitmapByName("Car1 ns r rr.bmp", &carBMP[i][DIRECTION_S][1][1]);
		LoadBitmapByName("Car1 ns l ll.bmp", &carBMP[i][DIRECTION_S][2][2]);

		LoadBitmapByName("Car1 we.bmp", &carBMP[i][DIRECTION_E][0][0]);
		LoadBitmapByName("Car1 we r.bmp", &carBMP[i][DIRECTION_E][1][0]);
		LoadBitmapByName("Car1 we l.bmp", &carBMP[i][DIRECTION_E][2][0]);
		LoadBitmapByName("Car1 we rr.bmp", &carBMP[i][DIRECTION_E][0][1]);
		LoadBitmapByName("Car1 we ll.bmp", &carBMP[i][DIRECTION_E][0][2]);
		LoadBitmapByName("Car1 we r rr.bmp", &carBMP[i][DIRECTION_E][1][1]);
		LoadBitmapByName("Car1 we l ll.bmp", &carBMP[i][DIRECTION_E][2][2]);
	}
}

void CWorld::AddCars()
{
	for(int i = 0; i < 9; i++)
	{
		CAutomobile a;
		a.automobileModel = 0;
		a.velociti = 3;
		a.distance = 60;
		a.startJunc = i;
		
		for(int k = 0; k < 4; k++)
		{
			if(juncs[i].neighbours[k] != -1)
			{
				a.endJunc = juncs[i].neighbours[k];
			}
		}
	
		a.juncs = &juncs;
		a.cars = &automobiles;
		a.y0 = AUTOMOBILE_STATE_RUN;

		automobiles.resize(i+1, a);
	}
}

void CWorld::DrawCars(CDC *pDC)
{
	for(int i = 0; i < automobiles.size(); i++)
	{
		CAutomobile *car = &automobiles[i];

		int x;
		int y;

		if((car->y0 == AUTOMOBILE_STATE_RUN) ||
			(car->y0 == AUTOMOBILE_STATE_WAIT_CAR) )
		{
			int distanceBetweenJuncs = sqrt(pow(juncs[car->endJunc].y - 
				juncs[car->startJunc].y,2)+pow(juncs[car->endJunc].x -
				juncs[car->startJunc].x,2));

			x = juncs[car->startJunc].x + car->distance*(juncs[car->endJunc].x-
				juncs[car->startJunc].x)/distanceBetweenJuncs;
			y = juncs[car->startJunc].y + car->distance*(juncs[car->endJunc].y-
				juncs[car->startJunc].y)/distanceBetweenJuncs;

			if(car->GetDirection() == DIRECTION_N)
			{
				y += 5;
				x += 5;

				DrawBitmap(pDC, &carBMP[car->automobileModel]
				[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
			if(car->GetDirection() == DIRECTION_E)
			{
				y += 5;
				x -= 5;

				DrawBitmap(pDC, &carBMP[car->automobileModel]
				[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
			if(car->GetDirection() == DIRECTION_S)
			{
				y -= 5;
				x -= 5;

				DrawBitmap(pDC, &carBMP[car->automobileModel]
				[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
			if(car->GetDirection() == DIRECTION_W)
			{
				y -= 5;
				x += 5;

				DrawBitmap(pDC, &carBMP[car->automobileModel]
				[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
		}

		if(	(car->y0 == AUTOMOBILE_STATE_NEAR_JUNC) ||
			(car->y0 == AUTOMOBILE_STATE_WAIT_CAR_NEAR_JUNC) ||
			(car->y0 == AUTOMOBILE_STATE_WAIT_ON_JANC))
		{
			int distanceBetweenJuncs = sqrt(pow(juncs[car->endJunc].y - 
				juncs[car->startJunc].y,2)+pow(juncs[car->endJunc].x - 
				juncs[car->startJunc].x,2));

			x = juncs[car->startJunc].x + car->distance*(juncs[car->endJunc].x-
				juncs[car->startJunc].x)/distanceBetweenJuncs;
			y = juncs[car->startJunc].y + car->distance*(juncs[car->endJunc].y-
				juncs[car->startJunc].y)/distanceBetweenJuncs;

			if(car->GetDirection() == DIRECTION_N)
			{
				y += 5;
				x += 5;

				if(car->lights)
					DrawBitmap(pDC, &carBMP[car->automobileModel]
					[car->GetDirection()][0][car->turnDirection],
						CPoint(x - 16,y - 16));
				else
					DrawBitmap(pDC, &carBMP[car->automobileModel]
					[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
			if(car->GetDirection() == DIRECTION_E)
			{
				y += 5;
				x -= 5;

				if(car->lights)
					DrawBitmap(pDC, &carBMP[car->automobileModel]
					[car->GetDirection()][0][car->turnDirection], 
						CPoint(x - 16,y - 16));
				else
					DrawBitmap(pDC, &carBMP[car->automobileModel]
					[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
			if(car->GetDirection() == DIRECTION_S)
			{
				y -= 5;
				x -= 5;

				if(car->lights)
					DrawBitmap(pDC, &carBMP[car->automobileModel]
					[car->GetDirection()][0][car->turnDirection], 
						CPoint(x - 16,y - 16));
				else
					DrawBitmap(pDC, &carBMP[car->automobileModel]
					[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
			if(car->GetDirection() == DIRECTION_W)
			{
				y -= 5;
				x += 5;

				if(car->lights)
					DrawBitmap(pDC, &carBMP[car->automobileModel]
					[car->GetDirection()][0][car->turnDirection], 
						CPoint(x - 16,y - 16));
				else
					DrawBitmap(pDC, &carBMP[car->automobileModel]
					[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
		}

		int direction = car->GetDirection();
		int td = car->turnDirection;
		int oldDirection;

		if(direction == DIRECTION_N)
		{
			if(td == DIRECTION_THROUGH)
				oldDirection = DIRECTION_N;
			if(td == DIRECTION_RIGHT)
				oldDirection = DIRECTION_W;
			if(td == DIRECTION_LEFT)
				oldDirection = DIRECTION_E;
		}
		if(direction == DIRECTION_E)
		{
			if(td == DIRECTION_THROUGH)
				oldDirection = DIRECTION_E;
			if(td == DIRECTION_RIGHT)
				oldDirection = DIRECTION_N;
			if(td == DIRECTION_LEFT)
				oldDirection = DIRECTION_S;
		}
		if(direction == DIRECTION_S)
		{
			if(td == DIRECTION_THROUGH)
				oldDirection = DIRECTION_S;
			if(td == DIRECTION_RIGHT)
				oldDirection = DIRECTION_E;
			if(td == DIRECTION_LEFT)
				oldDirection = DIRECTION_W;
		}
		if(direction == DIRECTION_W)
		{
			if(td == DIRECTION_THROUGH)
				oldDirection = DIRECTION_W;
			if(td == DIRECTION_RIGHT)
				oldDirection = DIRECTION_S;
			if(td == DIRECTION_LEFT)
				oldDirection = DIRECTION_N;
		}


		if(car->y0 == AUTOMOBILE_STATE_TURN_THROUGH)
		{
			int distanceBetweenJuncs = sqrt(pow(juncs[car->endJunc].y - 
				juncs[car->startJunc].y,2)+pow(juncs[car->endJunc].x - 
				juncs[car->startJunc].x,2));

			x = juncs[car->startJunc].x;
			y = juncs[car->startJunc].y;

			double dim = 8.0/10.0;

			if(oldDirection == DIRECTION_N)
			{
				y += 5;
				x += 5;

				y += 20;
				x -= 12;

				y -= 5*car->turnPosition*dim;
				x += 3*car->turnPosition*dim;

				DrawBitmap(pDC, &carBMP[car->automobileModel]
				[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
			if(oldDirection == DIRECTION_E)
			{
				y += 5;
				x -= 5;

				y -= 6;
				x -= 20;

				y += 1.5*car->turnPosition*dim;
				x += 5*car->turnPosition*dim;

				DrawBitmap(pDC, &carBMP[car->automobileModel]
				[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
			if(oldDirection == DIRECTION_S)
			{
				y -= 5;
				x -= 5;

				y -= 20;
				x += 12;

				y += 5*car->turnPosition*dim;
				x -= 3*car->turnPosition*dim;

				DrawBitmap(pDC, &carBMP[car->automobileModel]
				[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
			if(oldDirection == DIRECTION_W)
			{
				y -= 5;
				x += 5;

				y += 6;
				x += 20;

				y -= 1.5*car->turnPosition*dim;
				x -= 5*car->turnPosition*dim;

				DrawBitmap(pDC, &carBMP[car->automobileModel]
				[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
			}
		}
	
		if(car->y0 == AUTOMOBILE_STATE_TURN_RIGHT)
		{
			int distanceBetweenJuncs = sqrt(pow(juncs[car->endJunc].y - 
				juncs[car->startJunc].y,2)+pow(juncs[car->endJunc].x -
				juncs[car->startJunc].x,2));

			x = juncs[car->startJunc].x;
			y = juncs[car->startJunc].y;

			double dim = 8.0/10.0;

			if(car->turnPosition < 2)
			{

				if(oldDirection == DIRECTION_N)
				{
					y += 5;
					x += 5;

					y += 20;
					x -= 12;

					y -= 5*car->turnPosition*dim;
					x += 3*car->turnPosition*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_E)
				{
					y += 5;
					x -= 5;

					y -= 6;
					x -= 20;

					y += 1.5*car->turnPosition*dim;
					x += 5*car->turnPosition*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_S)
				{
					y -= 5;
					x -= 5;

					y -= 20;
					x += 12;

					y += 5*car->turnPosition*dim;
					x -= 3*car->turnPosition*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_W)
				{
					y -= 5;
					x += 5;

					y += 6;
					x += 20;

					y -= 1.5*car->turnPosition*dim;
					x -= 5*car->turnPosition*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][0], CPoint(x - 16,y - 16));
				}
			}

			if((car->turnPosition >= 2) && (car->turnPosition <= 4))
			{
				if(oldDirection == DIRECTION_N)
				{
					y += 5;
					x += 5;

					y += 20;
					x -= 12;

					y -= 5*2*dim;
					x += 3*2*dim;

					y -= 3*(car->turnPosition-2)*dim;
					x += 5*(car->turnPosition-2)*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][1][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][1][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_E)
				{
					y += 5;
					x -= 5;

					y -= 6;
					x -= 20;

					y += 1.5*2*dim;
					x += 5*2*dim;

					y += 5*(car->turnPosition-2)*dim;
					x += 1.5*(car->turnPosition-2)*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][1][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][1][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_S)
				{
					y -= 5;
					x -= 5;

					y -= 20;
					x += 12;

					y += 5*2*dim;
					x -= 3*2*dim;

					y += 3*(car->turnPosition-2)*dim;
					x -= 5*(car->turnPosition-2)*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][1][car->turnDirection],
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][1][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_W)
				{
					y -= 5;
					x += 5;

					y += 6;
					x += 20;

					y -= 1.5*2*dim;
					x -= 5*2*dim;

					y -= 5*(car->turnPosition-2)*dim;
					x += 1.5*(car->turnPosition-2)*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][1][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][1][0], CPoint(x - 16,y - 16));
				}
			}

			if(car->turnPosition > 4)
			{
				if(direction == DIRECTION_N)
				{
					y += 5;
					x += 5;

					y += 20;
					x -= 12;

					y -= 5*(car->turnPosition + 4)*dim;
					x += 3*(car->turnPosition + 4)*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[car->GetDirection()][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
				}
				if(direction == DIRECTION_E)
				{
					y += 5;
					x -= 5;

					y -= 6;
					x -= 20;

					y += 1.5*(car->turnPosition + 4)*dim;
					x += 5*(car->turnPosition + 4)*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[car->GetDirection()][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
				}
				if(direction == DIRECTION_S)
				{
					y -= 5;
					x -= 5;

					y -= 20;
					x += 12;

					y += 5*(car->turnPosition + 4)*dim;
					x -= 3*(car->turnPosition + 4)*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[car->GetDirection()][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
				}
				if(direction == DIRECTION_W)
				{
					y -= 5;
					x += 5;

					y += 6;
					x += 20;

					y -= 1.5*(car->turnPosition + 4)*dim;
					x -= 5*(car->turnPosition + 4)*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[car->GetDirection()][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
				}
			}
		}

		if(car->y0 == AUTOMOBILE_STATE_TURN_LEFT)
		{
			int distanceBetweenJuncs = sqrt(pow(juncs[car->endJunc].y - 
				juncs[car->startJunc].y,2)+pow(juncs[car->endJunc].x - 
				juncs[car->startJunc].x,2));

			x = juncs[car->startJunc].x;
			y = juncs[car->startJunc].y;

			double dim = 8.0/10.0;

			if(car->turnPosition < 2)
			{

				if(oldDirection == DIRECTION_N)
				{
					y += 5;
					x += 5;

					y += 20;
					x -= 12;

					y -= 5*car->turnPosition*dim;
					x += 3*car->turnPosition*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_E)
				{
					y += 5;
					x -= 5;

					y -= 6;
					x -= 20;

					y += 1.5*car->turnPosition*dim;
					x += 5*car->turnPosition*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_S)
				{
					y -= 5;
					x -= 5;

					y -= 20;
					x += 12;

					y += 5*car->turnPosition*dim;
					x -= 3*car->turnPosition*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_W)
				{
					y -= 5;
					x += 5;

					y += 6;
					x += 20;

					y -= 1.5*car->turnPosition*dim;
					x -= 5*car->turnPosition*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][0][0], CPoint(x - 16,y - 16));
				}

				//pDC->Ellipse(x-2,y-2,x+2,y+2);

			}

			if((car->turnPosition >= 2) && (car->turnPosition <= 8))
			{
				if(oldDirection == DIRECTION_N)
				{
					y += 5;
					x += 5;

					y += 20;
					x -= 12;

					y -= 5*2*dim;
					x += 3*2*dim;

					y -= 5*(car->turnPosition-2)*dim;
					x -= 3*(car->turnPosition-2)*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][2][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][2][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_E)
				{
					y += 5;
					x -= 5;

					y -= 6;
					x -= 20;

					y += 1.5*2*dim;
					x += 5*2*dim;

					y -= 1.5*(car->turnPosition-2)*dim;
					x += 5*(car->turnPosition-2)*dim;

					if(car->lights)
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][2][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, &carBMP[car->automobileModel]
						[oldDirection][2][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_S)
				{
					y -= 5;
					x -= 5;

					y -= 20;
					x += 12;

					y += 5*2*dim;
					x -= 3*2*dim;

					y += 5*(car->turnPosition-2)*dim;
					x += 3*(car->turnPosition-2)*dim;

					if(car->lights)
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[oldDirection][2][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[oldDirection][2][0], CPoint(x - 16,y - 16));
				}
				if(oldDirection == DIRECTION_W)
				{
					y -= 5;
					x += 5;

					y += 6;
					x += 20;

					y -= 1.5*2*dim;
					x -= 5*2*dim;

					y += 1.5*(car->turnPosition-2)*dim;
					x -= 5*(car->turnPosition-2)*dim;

					if(car->lights)
						DrawBitmap(pDC, &
						carBMP[car->automobileModel]
						[oldDirection][2][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[oldDirection][2][0], CPoint(x - 16,y - 16));
				}
			}

			if(car->turnPosition > 8)
			{
				if(direction == DIRECTION_N)
				{
					y += 5;
					x += 5;

					y += 20;
					x -= 12;

					y -= 5*(car->turnPosition)*dim;
					x += 3*(car->turnPosition)*dim;

					if(car->lights)
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[car->GetDirection()][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[car->GetDirection()][0][0], 
							CPoint(x - 16,y - 16));
				}
				if(direction == DIRECTION_E)
				{
					y += 5;
					x -= 5;

					y -= 6;
					x -= 20;

					y += 1.5*(car->turnPosition)*dim;
					x += 5*(car->turnPosition)*dim;

					if(car->lights)
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[car->GetDirection()][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[car->GetDirection()][0][0], 
							CPoint(x - 16,y - 16));
				}
				if(direction == DIRECTION_S)
				{
					y -= 5;
					x -= 5;

					y -= 20;
					x += 12;

					y += 5*(car->turnPosition)*dim;
					x -= 3*(car->turnPosition)*dim;

					if(car->lights)
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[car->GetDirection()][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[car->GetDirection()][0][0], 
							CPoint(x - 16,y - 16));
				}
				if(direction == DIRECTION_W)
				{
					y -= 5;
					x += 5;

					y += 6;
					x += 20;

					y -= 1.5*(car->turnPosition)*dim;
					x -= 5*(car->turnPosition)*dim;

					if(car->lights)
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[car->GetDirection()][0][car->turnDirection], 
							CPoint(x - 16,y - 16));
					else
						DrawBitmap(pDC, 
						&carBMP[car->automobileModel]
						[car->GetDirection()][0][0], CPoint(x - 16,y - 16));
				}

				//pDC->Ellipse(x-2,y-2,x+2,y+2);

			}

		}
		car->point.x = x;
		car->point.y = y;

		//pDC->Ellipse(x-2,y-2,x+2,y+2);
	}
}


void CWorld::LoadMap(CString mapName)
{
	CFile file(mapName, CFile::modeRead);
	LoadBMP(&file, &map);
	file.Close();
}

void CWorld::DrawMap(CDC *pDC)
{
	int width = map.GetBitmapDimension().cx;
	int height = map.GetBitmapDimension().cy;

	pDC->DrawState(CPoint(0, 0),CSize(width, height),&map,DST_BITMAP);
}

void CWorld::LoadJuncs(CString juncsName)
{
	LoadBitmapByName("Junc X bg.bmp", &juncBG[JUNC_TYPE_X]);
	LoadBitmapByName("Junc X tl r.bmp", &juncTLRed[JUNC_TYPE_X]);
	LoadBitmapByName("Junc X tl g.bmp", &juncTLGreen[JUNC_TYPE_X]);
	LoadBitmapByName("Junc X tl y.bmp", &juncTLYellow[JUNC_TYPE_X]);

	LoadBitmapByName("Junc T s bg.bmp", &juncBG[JUNC_TYPE_T_S]);
	LoadBitmapByName("Junc T s tl r.bmp", &juncTLRed[JUNC_TYPE_T_S]);
	LoadBitmapByName("Junc T s tl g.bmp", &juncTLGreen[JUNC_TYPE_T_S]);
	LoadBitmapByName("Junc T s tl y.bmp", &juncTLYellow[JUNC_TYPE_T_S]);

	LoadBitmapByName("Junc T w bg.bmp", &juncBG[JUNC_TYPE_T_W]);
	LoadBitmapByName("Junc T w tl r.bmp", &juncTLRed[JUNC_TYPE_T_W]);
	LoadBitmapByName("Junc T w tl g.bmp", &juncTLGreen[JUNC_TYPE_T_W]);
	LoadBitmapByName("Junc T w tl y.bmp", &juncTLYellow[JUNC_TYPE_T_W]);


	CStdioFile file(juncsName, CStdioFile::modeRead);

	CString data;
	file.ReadString(data);

	int juncsNumber = atoi(data);

	for(int i = 0; i < juncsNumber; i++)
	{
		file.ReadString(data);
		char *p = data.GetBuffer(); 
		CJunction junc;

		int j = 0;
		while(p[j] != ' '){j++;}
		j++;
		junc.x = atoi(p + j);
		while(p[j] != ' '){j++;}
		j++;
		junc.y = atoi(p + j);

		int n = 4;

		for(int k = 0; k < n; k++)
		{
			while(p[j] != ' '){j++;}
			j++;

			if(p[j] != '#')
			{
				junc.neighbours[k] = atoi(p + j);
			}
			else
			{
				junc.neighbours[k] = -1;
			}
		}

		if((junc.neighbours[0] != -1) &&
			(junc.neighbours[1] != -1) &&
			(junc.neighbours[2] != -1) &&
			(junc.neighbours[3] != -1))
		{
			junc.type = JUNC_TYPE_X;
		}
		else if((junc.neighbours[0] == -1) &&
			(junc.neighbours[1] != -1) &&
			(junc.neighbours[2] != -1) &&
			(junc.neighbours[3] != -1))
		{
			junc.type = JUNC_TYPE_T_S;
		}
		else if((junc.neighbours[0] != -1) &&
			(junc.neighbours[1] != -1) &&
			(junc.neighbours[2] == -1) &&
			(junc.neighbours[3] != -1))
		{
			junc.type = JUNC_TYPE_T_N;
		}
		else if((junc.neighbours[0] != -1) &&
			(junc.neighbours[1] == -1) &&
			(junc.neighbours[2] != -1) &&
			(junc.neighbours[3] != -1))
		{
			junc.type = JUNC_TYPE_T_E;
		}
		else if((junc.neighbours[0] != -1) &&
			(junc.neighbours[1] != -1) &&
			(junc.neighbours[2] != -1) &&
			(junc.neighbours[3] == -1))
		{
			junc.type = JUNC_TYPE_T_W;
		}
		else
		{
			junc.type = JUNC_TYPE_NO_TL;
		}

		juncs.resize(i+1, junc);
	}

	file.Close();
}

void CWorld::DrawJuncsBG(CDC *pDC)
{
	for(int i = 0; i < juncs.size(); i++)
	{
		if(juncs[i].type == JUNC_TYPE_X)
		{
			DrawBitmap(pDC, &juncBG[juncs[i].type], 
				CPoint(juncs[i].x - 60, juncs[i].y - 55));
		}
		if(juncs[i].type == JUNC_TYPE_T_S)
		{
			DrawBitmap(pDC, &juncBG[juncs[i].type], 
				CPoint(juncs[i].x - 62, juncs[i].y - 46));
		}
		if(juncs[i].type == JUNC_TYPE_T_W)
		{
			DrawBitmap(pDC, &juncBG[juncs[i].type], 
				CPoint(juncs[i].x - 56, juncs[i].y - 55));
		}
	}
}

void CWorld::DrawJuncsTL(CDC *pDC)
{
	for(int i = 0; i < juncs.size(); i++)
	{
		if(juncs[i].type != JUNC_TYPE_NO_TL)
		{
			if(juncs[i].y1 == JUNC_STATE_GREEN)
				DrawBitmap(pDC, &juncTLGreen[juncs[i].type], 
				CPoint(juncs[i].x - 39, juncs[i].y - 58));

			if(juncs[i].y1 == JUNC_STATE_RED)
				DrawBitmap(pDC, &juncTLRed[juncs[i].type], 
				CPoint(juncs[i].x - 39, juncs[i].y - 58));

			if((juncs[i].y1 == JUNC_STATE_YELLOW_TO_GREEN) || 
				(juncs[i].y1 == JUNC_STATE_YELLOW_TO_RED))
				DrawBitmap(pDC, &juncTLYellow[juncs[i].type], 
				CPoint(juncs[i].x - 39, juncs[i].y - 58));
		}
	}
}

void CWorld::LoadBMP(CFile *file, CBitmap *bitmap)
{
	BITMAPFILEHEADER bFH;
	BITMAPINFOHEADER bIH;

	file->Read(&bFH, sizeof(bFH));
	file->Read(&bIH, sizeof(bIH));

	int bitmapWidth = bIH.biWidth;
	int bitmapHeight = bIH.biHeight;

	BYTE* loadBitmapData = new BYTE[bIH.biSizeImage];
	BYTE* bitmapData = new BYTE[bitmapWidth*bitmapHeight*4];

	file->Read(loadBitmapData, bIH.biSizeImage);

	int addBytesNumber = bitmapWidth%4;

	for(int i = 0; i < bitmapHeight; i++)
	{
		for(int j = 0; j < bitmapWidth; j++)
		{
			bitmapData[4*bitmapWidth*(bitmapHeight - i - 1) + 4*j] = 
				loadBitmapData[(3*bitmapWidth + addBytesNumber)*i + 3*j];
			bitmapData[4*bitmapWidth*(bitmapHeight - i - 1) + 4*j + 1] = 
				loadBitmapData[(3*bitmapWidth + addBytesNumber)*i + 3*j + 1];
			bitmapData[4*bitmapWidth*(bitmapHeight - i - 1) + 4*j + 2] = 
				loadBitmapData[(3*bitmapWidth + addBytesNumber)*i + 3*j + 2];

			if((bitmapData[4*bitmapWidth*(bitmapHeight-i-1)+4*j] == 0) &&
				(bitmapData[4*bitmapWidth*(bitmapHeight-i-1)+4*j+1] == 0) &&
				(bitmapData[4*bitmapWidth*(bitmapHeight-i-1)+ 4*j + 2] == 0))
			{
				bitmapData[4*bitmapWidth*(bitmapHeight - i - 1) + 4*j + 3] = 0;
			}
			else
			{
				bitmapData[4*bitmapWidth*(bitmapHeight - i - 1) + 4*j + 3] =
					255;
			}
		}
	}

	bitmap->CreateBitmap(bitmapWidth, bitmapHeight,1,32,bitmapData);

	delete[] bitmapData;
	delete[] loadBitmapData;
}

void CWorld::LoadBitmapByName(CString fileName, CBitmap *bitmap)
{
	CFile file(fileName, CFile::modeRead);
	LoadBMP(&file, bitmap);
	file.Close();
}

void CWorld::DrawBitmap(CDC* pBaseDC, CBitmap *bitmap, CPoint point)
{
	CSize size = GetBitmapSize(bitmap);

	CDC dc;
	dc.CreateCompatibleDC(pBaseDC);;
	CDC *pDC = &dc;

	pDC->SelectObject(bitmap);

	BLENDFUNCTION blend;
	/*   blend.BlendOp = AC_SRC_OVER;
	blend.BlendFlags = 0;
	blend.SourceConstantAlpha = 0x7f;  // half of 0xff = 50% transparency
	blend.AlphaFormat = 0;             // ignore source alpha channel
	*/
	blend.BlendOp = AC_SRC_OVER;
	blend.BlendFlags = 0;
	blend.AlphaFormat = AC_SRC_ALPHA;   
	blend.SourceConstantAlpha = 0xff;   

	pBaseDC->AlphaBlend(point.x, point.y, 
		size.cx, size.cy, 
		pDC, 0, 0, size.cx, size.cy, blend);

	pDC->DeleteDC();
}

CSize CWorld::GetBitmapSize(CBitmap *bitmap)
{
	BITMAP b;
	bitmap->GetBitmap(&b);
	return CSize(b.bmWidth, b.bmHeight);

}